

Imports System.Runtime.InteropServices
Imports System.Text

' --------------------------------------------------------------------- a device capable of sending MIDI messages
Public NotInheritable Class OutputDevice
	Inherits OutputDeviceBase

    <DllImport("winmm.dll")> _
    Private Shared Function midiOutOpen(ByRef handle As Integer, ByVal deviceID As Integer, ByVal proc As MidiOutProc, ByVal instance As Integer, ByVal flags As Integer) As Integer
    End Function

    <DllImport("winmm.dll")> _
    Private Shared Function midiOutClose(ByVal handle As Integer) As Integer
    End Function


    Private m_midiOutProc As MidiOutProc
	Private m_runningStatusEnabled As Boolean = False
    Private runningStatus As Integer = 0

    Public IsOpened As Boolean = False

    Public Sub New(ByVal deviceID As Integer)
        MyBase.New(deviceID)
        m_midiOutProc = AddressOf HandleMessage

        Dim result As Integer = midiOutOpen(m_handle, deviceID, m_midiOutProc, 0, CALLBACK_FUNCTION)
        If result = MidiDeviceException.MMSYSERR_NOERROR Then
            IsOpened = True
        End If
    End Sub


	Protected Overloads Overrides Sub Dispose(disposing As Boolean)
		If disposing Then
			SyncLock lockObject
				Reset()
                Dim result As Integer = midiOutClose(m_handle)
			End SyncLock
		Else
            midiOutReset(m_handle)
            midiOutClose(m_handle)
        End If
        IsOpened = False
		MyBase.Dispose(disposing)
	End Sub


	Public Overloads Overrides Sub Close()
        If IsDisposed Then
            Return
        End If
        Dispose(True)
    End Sub


	Public Overloads Overrides Sub Reset()
        If IsDisposed Then
            Throw New ObjectDisposedException(Me.[GetType]().Name)
        End If
        runningStatus = 0
        MyBase.Reset()
    End Sub


	Public Overloads Overrides Sub Send(message As ChannelMessage)
        If IsDisposed Then
            Throw New ObjectDisposedException(Me.[GetType]().Name)
        End If
        SyncLock lockObject
            ' If running status is enabled.
            If m_runningStatusEnabled Then
                ' If the message's status value matches the running status.
                If message.Status = runningStatus Then
                    ' Send only the two data bytes without the status byte.
                    Send(message.Message >> 8)
                Else
                    ' Else the message's status value does not match the running
                    ' status.
                    ' Send complete message with status byte.
                    Send(message.Message)

                    ' Update running status.
                    runningStatus = message.Status
                End If
            Else
                ' Else running status has not been enabled.
                Send(message.Message)
            End If
        End SyncLock
    End Sub

	Public Overloads Overrides Sub Send(message As SysExMessage)
		' System exclusive cancels running status.
		runningStatus = 0
		MyBase.Send(message)
	End Sub

	Public Overloads Overrides Sub Send(message As SysCommonMessage)
        If IsDisposed Then
            Throw New ObjectDisposedException(Me.[GetType]().Name)
        End If
        ' System common cancels running status.
        runningStatus = 0
        MyBase.Send(message)
    End Sub

    Public Property RunningStatusEnabled() As Boolean
        Get
            Return m_runningStatusEnabled
        End Get
        Set(ByVal value As Boolean)
            m_runningStatusEnabled = Value
            ' Reset running status.
            runningStatus = 0
        End Set
    End Property

End Class


Public Class OutputDeviceException
    Inherits MidiDeviceException

    <DllImport("winmm.dll")> _
    Private Shared Function midiOutGetErrorText(ByVal errCode As Integer, ByVal message As StringBuilder, ByVal sizeOfMessage As Integer) As Integer
    End Function

    Private m_message As New StringBuilder(128)

    Public Sub New(ByVal errCode As Integer)
        MyBase.New(errCode)
        ' Get error message.
        midiOutGetErrorText(errCode, m_message, m_message.Capacity)
    End Sub

    Public Overloads Overrides ReadOnly Property Message() As String
        Get
            Return m_message.ToString()
        End Get
    End Property

End Class
