

Imports System.Text

Public Class MetaTextBuilder
    Implements IMessageBuilder

    Private m_text As String
    Private m_type As MetaType = MetaType.Text
    Private m_result As MetaMessage = Nothing
    Private changed As Boolean = True

    Public Sub New()
        m_text = String.Empty
    End Sub

    Public Sub New(ByVal type As MetaType)
        If Not IsTextType(type) Then
            Throw New ArgumentException("Not text based meta message type.", "message")
        End If
        Me.m_text = String.Empty
    End Sub

    Public Sub New(ByVal type As MetaType, ByVal text As String)
        If Not IsTextType(type) Then
            Throw New ArgumentException("Not text based meta message type.", "message")
        End If

        Me.m_type = type

        If text IsNot Nothing Then
            Me.m_text = text
        Else
            Me.m_text = String.Empty
        End If
    End Sub

    Public Sub New(ByVal message As MetaMessage)
        Initialize(message)
    End Sub


    Public Sub Initialize(ByVal message As MetaMessage)
        If Not IsTextType(message.MetaType) Then
            Throw New ArgumentException("Not text based meta message.", "message")
        End If
        Dim encoding As New ASCIIEncoding()
        m_text = encoding.GetString(message.GetBytes())
        Me.m_type = message.MetaType
    End Sub

    Private Function IsTextType(ByVal type As MetaType) As Boolean
        Dim result As Boolean
        If type = MetaType.Copyright OrElse type = MetaType.CuePoint OrElse type = MetaType.DeviceName OrElse type = MetaType.InstrumentName OrElse type = MetaType.Lyric OrElse type = MetaType.Marker OrElse type = MetaType.ProgramName OrElse type = MetaType.Text OrElse type = MetaType.TrackName Then
            result = True
        Else
            result = False
        End If
        Return result
    End Function


    Public Property Text() As String
        Get
            Return m_text
        End Get
        Set(ByVal value As String)
            If value IsNot Nothing Then
                m_text = value
            Else
                m_text = String.Empty
            End If
            changed = True
        End Set
    End Property

    Public Property Type() As MetaType
        Get
            Return m_type
        End Get
        Set(ByVal value As MetaType)
            If Not IsTextType(value) Then
                Throw New ArgumentException("Not text based meta message type.", "message")
            End If
            m_type = value
            changed = True
        End Set
    End Property

    Public ReadOnly Property Result() As MetaMessage
        Get
            Return m_result
        End Get
    End Property

    Public Sub Build() Implements IMessageBuilder.Build
        If changed Then
            Dim encoding As New ASCIIEncoding()
            Dim data As Byte() = encoding.GetBytes(m_text)
            m_result = New MetaMessage(Type, data)
            changed = False
        End If
    End Sub

End Class
