


Public Class SongPositionPointerBuilder
    Implements IMessageBuilder

    Private Const TicksPer16thNote As Integer = 6
    ' -------------------------------------------------------- for packing and unpacking the song position
    Private Const Shift As Integer = 7
    ' -------------------------------------------------------- for packing and unpacking the song position
    Private Const Mask As Integer = 127
    ' -------------------------------------------------------- scale used for converting from the song position
    '                                                           to the position in ticks
    Private tickScale As Integer
    ' -------------------------------------------------------- pulses per quarter note resolution
    Private m_ppqn As Integer
    ' -------------------------------------------------------- used for building the SysCommonMessage 
    '                                                           to represent the song position pointer
    Private builder As SysCommonMessageBuilder


    Public Sub New()
        builder = New SysCommonMessageBuilder()
        builder.Type = SysCommonType.SongPositionPointer
        Ppqn = PpqnClock.PpqnMinValue
    End Sub

    Public Sub New(ByVal message As SysCommonMessage)
        builder = New SysCommonMessageBuilder()
        builder.Type = SysCommonType.SongPositionPointer
        Initialize(message)
        Ppqn = PpqnClock.PpqnMinValue
    End Sub

    Public Sub Initialize(ByVal message As SysCommonMessage)
        If message Is Nothing Then
            Throw New ArgumentNullException("message")
        ElseIf message.SysCommonType <> SysCommonType.SongPositionPointer Then
            Throw New ArgumentException("Message is not a song position pointer message.")
        End If
        builder.Initialize(message)
    End Sub


    Public Property PositionInTicks() As Integer
        Get
            Return SongPosition * tickScale * TicksPer16thNote
        End Get
        Set(ByVal value As Integer)
            If value < 0 Then
                Throw New ArgumentOutOfRangeException("PositionInTicks", value, "Position in ticks out of range.")
            End If
            SongPosition = value \ (tickScale * TicksPer16thNote)
        End Set
    End Property

    ' -------------------------------------------------------------------- PulsesPerQuarterNote
    Public Property Ppqn() As Integer
        Get
            Return m_ppqn
        End Get
        Set(ByVal value As Integer)
            If value Mod PpqnClock.PpqnMinValue <> 0 Then
                Throw New ArgumentException("Invalid pulses per quarter note value.")
            End If
            m_ppqn = value
            tickScale = m_ppqn \ PpqnClock.PpqnMinValue
        End Set
    End Property

    Public Property SongPosition() As Integer
        Get
            Return (builder.Data2 << Shift) Or builder.Data1
        End Get
        Set(ByVal value As Integer)
            If value < 0 Then
                Throw New ArgumentOutOfRangeException("SongPosition", value, "Song position pointer out of range.")
            End If
            builder.Data1 = value And Mask
            builder.Data2 = value >> Shift
        End Set
    End Property

    ' -------------------------------------------------------------------- the built song position pointer message
    Public ReadOnly Property Result() As SysCommonMessage
        Get
            Return builder.Result
        End Get
    End Property

    Public Sub Build() Implements IMessageBuilder.Build
        builder.Build()
    End Sub

End Class
