

' ------------------------------------------------------ dispatches IMidiMessages to their corresponding sink
Public Class MessageDispatcher

    Public Event ChannelMessageDispatched As EventHandler(Of ChannelMessageEventArgs)
    Public Event SysExMessageDispatched As EventHandler(Of SysExMessageEventArgs)
    Public Event SysCommonMessageDispatched As EventHandler(Of SysCommonMessageEventArgs)
    Public Event SysRealtimeMessageDispatched As EventHandler(Of SysRealtimeMessageEventArgs)
    Public Event MetaMessageDispatched As EventHandler(Of MetaMessageEventArgs)


    Public Sub Dispatch(ByVal message As IMidiMessage)
        If message Is Nothing Then
            Throw New ArgumentNullException("message")
        End If

        Select Case message.MessageType
            Case MessageType.Channel
                OnChannelMessageDispatched(New ChannelMessageEventArgs(DirectCast(message, ChannelMessage)))

            Case MessageType.SystemExclusive
                OnSysExMessageDispatched(New SysExMessageEventArgs(DirectCast(message, SysExMessage)))

            Case MessageType.Meta
                OnMetaMessageDispatched(New MetaMessageEventArgs(DirectCast(message, MetaMessage)))

            Case MessageType.SystemCommon
                OnSysCommonMessageDispatched(New SysCommonMessageEventArgs(DirectCast(message, SysCommonMessage)))

            Case MessageType.SystemRealtime
                Select Case DirectCast(message, SysRealtimeMessage).SysRealtimeType
                    Case SysRealtimeType.MsgActiveSense
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgActiveSense)

                    Case SysRealtimeType.MsgClock
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgClock)

                    Case SysRealtimeType.MsgContinue
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgContinue)

                    Case SysRealtimeType.MsgReset
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgReset)

                    Case SysRealtimeType.MsgStart
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgStart)

                    Case SysRealtimeType.MsgStop
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgStop)

                    Case SysRealtimeType.MsgTick
                        OnSysRealtimeMessageDispatched(SysRealtimeMessageEventArgs.MsgTick)

                End Select

        End Select

    End Sub

    Protected Overridable Sub OnChannelMessageDispatched(ByVal e As ChannelMessageEventArgs)
        RaiseEvent ChannelMessageDispatched(Me, e)
    End Sub

    Protected Overridable Sub OnSysExMessageDispatched(ByVal e As SysExMessageEventArgs)
        RaiseEvent SysExMessageDispatched(Me, e)
    End Sub

    Protected Overridable Sub OnSysCommonMessageDispatched(ByVal e As SysCommonMessageEventArgs)
        RaiseEvent SysCommonMessageDispatched(Me, e)
    End Sub

    Protected Overridable Sub OnSysRealtimeMessageDispatched(ByVal e As SysRealtimeMessageEventArgs)
        RaiseEvent SysRealtimeMessageDispatched(Me, e)
    End Sub

    Protected Overridable Sub OnMetaMessageDispatched(ByVal e As MetaMessageEventArgs)
        RaiseEvent MetaMessageDispatched(Me, e)
    End Sub

End Class
